#ifndef TT_COMMAND_LINE_ANALYZER_H_
#define TT_COMMAND_LINE_ANALYZER_H_

#include <string>
#include <vector>
#include <set>

// -- TtCommandLineOption ------------------------------------------------
class TtCommandLineOption {
  friend class TtCommandLineOptionAnalyzer;

public:
  explicit TtCommandLineOption( const std::string& main_option, unsigned int operand_count, bool is_indispensable = false );
  virtual ~TtCommandLineOption();

  const std::string& GetMainOption( void ) const;
  void AddOptionString( const std::string& option );
  const std::set<std::string>& GetOptionStrings( void ) const;

  void SetIndispensable( bool flag );
  bool IsIndispensable( void ) const;

  bool IsSpecified( void ) const;

  const std::vector<std::string>& GetOperand( void ) const;

private:
  const std::string        main_option_;
  std::set<std::string>    option_strings_;
  std::vector<std::string> operand_;
  unsigned int             operand_count_;
  bool                     is_indispensable_;
  bool                     is_specified_;
};

// -- TtCommandLineOptionBoolean -----------------------------------
class TtCommandLineOptionBoolean : public TtCommandLineOption {
public:
  explicit TtCommandLineOptionBoolean( const std::string& option, bool is_indispensable = false );
};

// -- TtCommandLineOptionSingleOperand -----------------------------------
class TtCommandLineOptionSingleOperand : public TtCommandLineOption {
public:
  explicit TtCommandLineOptionSingleOperand( const std::string& option, bool is_indispensable = false );

  const std::string& GetSingleOperand( void ) const;
};

// -- TtCommandLineOptionAnalyzer ----------------------------------------
class TtCommandLineOptionAnalyzer {
public:
  explicit TtCommandLineOptionAnalyzer( void );
  ~TtCommandLineOptionAnalyzer();

  void Regist( TtCommandLineOption& option );

  std::vector<std::string> AnalyzeForConsole( unsigned int argc, const char* const* argv );
  std::vector<std::string> AnalyzeForWindows( const std::string& args_string );

private:
  std::vector<std::string> Analyze( const std::vector<std::string> args );

private:
  std::set<TtCommandLineOption*> options_;
};

#endif // TT_COMMAND_LINE_ANALYZER_H_
