#ifndef TT_LOGGER_H_
#define TT_LOGGER_H_

#include <stdio.h>
#include <string>

#include "tt_critical_section.h"
#include "tt_temporaly_array.h"

class TtLogger;

// -- TtLogFlusher -------------------------------------------------------
class TtLogFlusher {
private:
  friend class TtLogger;
  explicit TtLogFlusher( void ) {};
};

// -- TtLogger -----------------------------------------------------------
class TtLogger {
public:
  static TtLogFlusher FLUSH;

  explicit TtLogger( void );
  explicit TtLogger( const std::string& filename );
  virtual ~TtLogger();

  const std::string& GetFileName( void );

  void Open( void );
  void Close( void );

  TtLogger& operator <<( int value );
  TtLogger& operator <<( unsigned int value );
  TtLogger& operator <<( const char* value );
  TtLogger& operator <<( const std::string& value );
  void operator <<( TtLogFlusher& flusher );

  void Put( const std::string& value );

protected:
  virtual void Prefix( std::string& str );
  virtual void Postfix( std::string& str );

private:
  std::string       filename_;
  FILE*             file_stream_;
  TtCriticalSection lock_;
  std::string       buffer_;
};


// -- TtLoggerStandard ---------------------------------------------------
class TtLoggerStandard : public TtLogger {
public:
  explicit TtLoggerStandard( void );
  explicit TtLoggerStandard( const std::string& filename );

protected:
  virtual void Prefix( std::string& str );
  virtual void Postfix( std::string& str );
};

#endif // TT_LOGGER_H_
