#include <errno.h>
#include <stdio.h>

#include "ttl_define.h"
#include "tt_util.h"


bool
TtUtil::StringToInteger( const std::string& str, int* ret, unsigned int base )
{
  if ( str.empty() ) {
    return false;
  }
  char* endptr;
  errno = 0;
  *ret = strtol( str.c_str(), &endptr, base );
  return *endptr == '\0' && errno == 0;
}

bool
TtUtil::StringToDouble( const std::string& str, double* ret )
{
  if ( str.empty() ) {
    return false;
  }
  char* endptr;
  errno = 0;
  *ret = strtod( str.c_str(), &endptr );
  return *endptr == '\0' && NOT( *ret == 0.0 && endptr != str.c_str() ) && errno == 0;
}


std::string
TtUtil::StringReplace( const std::string& source, const std::string& pattern, const std::string& replacement )
{
  unsigned int pos = 0;
  std::string tmp;
  for (;;) {
    unsigned int result = source.find( pattern, pos );
    if ( result == std::string::npos ) {
      break;
    }
    tmp.append( source, pos, result - pos );
    tmp.append( replacement );
    pos = result + pattern.size();
  }
  tmp.append( source, pos, source.size() - pos );
  return tmp;
}


std::string
TtUtil::GetWindowsSystemErrorMessage( unsigned int error_number )
{
  char buf[1024 * 16];
  int ret = FormatMessage(
    FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_SYSTEM,
    NULL, error_number, MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ),
    buf, sizeof( buf ), NULL );
  if ( ret == 0 ) {
    return "FormatMessage error.";
  }
  return std::string( buf );
}


std::string
TtUtil::BccCompilerDateMacroToNormalString( const std::string& date )
{
  std::string tmp;
  tmp.append( date.substr( date.size() - 4, 4 ) );

  tmp.append( "/" );

  std::string month = date.substr( 0, 3 );
  tmp.append( month == "Jan" ? "01" :
              month == "Feb" ? "02" :
              month == "Mar" ? "03" :
              month == "Apr" ? "04" :
              month == "May" ? "05" :
              month == "Jun" ? "06" :
              month == "Jul" ? "07" :
              month == "Aug" ? "08" :
              month == "Sep" ? "09" :
              month == "Oct" ? "10" :
              month == "Nov" ? "11" :
              month == "Dec" ? "12" : "??" );

  tmp.append( "/" );

  std::string day = date.substr( 4, 2 );
  if ( day[0] == ' ' ) {
    day[0] = '0';
  }
  tmp.append( day );
  return tmp;
}


void
TtUtil::MessageBoxForDebug( const char* type, const char* obj_name, void* obj, const char* file, int line )
{
  char fmt[256];
  char buf[1024 * 16];
  sprintf( fmt, "%%s(%%d) : %%s = %s\n", type );
  sprintf( buf, fmt, file, line, obj_name, obj );
  MessageBox( NULL, buf, "DEBUG", 0 );
}
